// *****************************************************************
// ****                      Synth CORE B                       ****
// ****                  www.tolaemon.com/scoreb                ****
// ****                       Source code                       ****
// ****                       13/11/2011                        ****
// ****                     Jordi Bartolome                     ****
// ****                                                         ****
// **** IMPORTANT:                                              ****
// **** Using parts of this code means accepting all conditions ****
// **** exposed on the web: www.tolaemon.com/scoreb             ****
// ****                                                         ****
// **** Parts of this code have been taken from the demo source ****
// **** code of EVK1100 Atmel's evaluation board				****
// *****************************************************************

#include <avr32/io.h>
#include "board.h"
#include "gpio.h"
#include "usart.h"
#include "adc.h"

// scheduler includes.
#include "FreeRTOS.h"
#include "queue.h"
#include "task.h"

// synth core B includes
#include "defines.h"
#include "adc_converters.h"

// ADCs control structures

struct {
	portCHAR initialized; // true if the ADCs converter is initialized and running
	unsigned portSHORT value[ADC_CONVERTERS_NUM_CHANNELS];//array with the values of each converter . Despite adc_get_value returns unsgined long, ATMELs ADCs have ony 10bits ressolution so a unsigned short is enough to store its value.
	unsigned portCHAR enabled[ADC_CONVERTERS_NUM_CHANNELS];//array with a flag indicating if the adc is available to use
	xTaskHandle task_handler; // the handler to the FreeRTOS adc converters task
	unsigned portSHORT sample_period;// the number of miliseconds between each conversion cicle
} adc_converters_control;


static const gpio_map_t ADC_GPIO_MAP =
{
  {AVR32_ADC_AD_1_PIN, AVR32_ADC_AD_1_FUNCTION},
  {AVR32_ADC_AD_2_PIN, AVR32_ADC_AD_2_FUNCTION}, // TODO I must delete this ADC which is not on the board.
  {AVR32_ADC_AD_3_PIN, AVR32_ADC_AD_3_FUNCTION},
  {AVR32_ADC_AD_4_PIN, AVR32_ADC_AD_4_FUNCTION},
  {AVR32_ADC_AD_5_PIN, AVR32_ADC_AD_5_FUNCTION},
  {AVR32_ADC_AD_6_PIN, AVR32_ADC_AD_6_FUNCTION},
  {AVR32_ADC_AD_7_PIN, AVR32_ADC_AD_7_FUNCTION}
};

volatile avr32_adc_t * adc = &AVR32_ADC; // ADC IP registers address


// Synth Core  13-11-2011  Tolaemon
// Task which manages the ADCs conversion
static void ADC_CONVERTERS_task(void *pvParameters ){
	unsigned portCHAR uc_aux;

	while (1){

	    // starts the conversion on the ADCs
	    adc_start(adc);

	    // datasheets says this about conversion time
	    // 1. Corresponds to 13 clock cycles at 5 MHz: 3 clock cycles for track and hold acquisition time and 10 clock cycles for conversion.
	    // 2. Corresponds to 15 clock cycles at 8 MHz: 5 clock cycles for track and hold acquisition time and 10 clock cycles for conversion.
	    vTaskDelay(1); // so 1 ms is much more than the conversion time but is OK fou our purposes

	    // reads the value of each ADC converter
		for (uc_aux=0;uc_aux<ADC_CONVERTERS_NUM_CHANNELS;uc_aux++){

			// before reading check that the ADC CONVERTER channel is enabled
			if (adc_converters_control.enabled[uc_aux]==TRUE){
				adc_converters_control.value[uc_aux] = (unsigned short) adc_get_value(adc, (unsigned short)uc_aux);
			}//if

		}//for

		// wait for the programmed lapse of time between conversions
		vTaskDelay(adc_converters_control.sample_period);

	}//while

}//ADC_CONVERTERS_task



// Synth Core  13-11-2011  Tolaemon
// Function: void ADC_CONVERTERS_reset()
// Function wich resets ADC_CONVERTERS control vars. It is used when
// intializing the system to grant all vars are on "initial conditions"
// Receives:
// Returns:
void ADC_CONVERTERS_reset(){
	unsigned portCHAR uc_aux;

	adc_converters_control.initialized=FALSE;
	adc_converters_control.task_handler=NULL;
	adc_converters_control.sample_period=0;

	for (uc_aux=0;uc_aux<ADC_CONVERTERS_NUM_CHANNELS;uc_aux++){
		adc_converters_control.value[uc_aux]=0;
		adc_converters_control.enabled[uc_aux]=FALSE;
	}//for

}// ADC_CONVERTERS_reset



// Synth Core  13-11-2011  Tolaemon
// Function: void ADC_CONVERTERS_init()
// Function which creates and prepares the resources needed to
// use the ADC CONVERTERS
// Receives:
// Returns:
void ADC_CONVERTERS_init(unsigned portSHORT sample_period){
	unsigned portCHAR uc_aux;


	// assign and enable GPIO pins to the ADC function.
	gpio_enable_module(ADC_GPIO_MAP, sizeof(ADC_GPIO_MAP) / sizeof(ADC_GPIO_MAP[0]));

	// configure ADC
	adc_configure(adc);

	// initializes de flags that indicate if an ADC channel hardware is available
	for (uc_aux=0;uc_aux<ADC_CONVERTERS_NUM_CHANNELS;uc_aux++){
		adc_converters_control.enabled[uc_aux]=TRUE;
	}//for
	adc_converters_control.enabled[2]=FALSE; // ADC channel 2 is not available because it is used by another peripheral
	adc_converters_control.sample_period = sample_period;

	// enables each of the ADC channels ( sets the bits that enable specified ADC channels )
	for (uc_aux=0;uc_aux<ADC_CONVERTERS_NUM_CHANNELS;uc_aux++){
		if (adc_converters_control.enabled[uc_aux]==TRUE){
			adc_enable(adc,uc_aux);
		}//if
	}//for

	// create the midi task to start processing the bytes
	xTaskCreate( ADC_CONVERTERS_task ,  (const signed portCHAR *)"ADC CONV task",  ADC_TASK_STACK_SIZE ,  NULL ,  ADC_TASK_PRIORITY ,&adc_converters_control.task_handler);

	adc_converters_control.initialized=TRUE;

}//ADC_CONVERTERS_init



// Synth Core  13-11-2011  Tolaemon
// Function: void ADC_CONVERTERS_close()
// Function which closes ADC CONVERTERS resources
// Receives:
// channel_number: from 0 to ADC_CONVERTERS_NUM_CAHNNELS-1 with the
// number of ADC channel we want to read
// Returns:
//   By value: the value of the specified ADC channel
// Note:
//  if the function is called but the channel number is wrong or ADCs
//  CONVERTER has not been initialized it returns 0
unsigned portSHORT ADC_CONVERTERS_read_value(unsigned portCHAR channel_number){
	unsigned long return_value=0;

	// before reading the ADC channel we check that the driver has been previously
	// initalized and the channel_number corresponds to an available ADC channel
	if ( (adc_converters_control.initialized==TRUE)
			&&(channel_number<ADC_CONVERTERS_NUM_CHANNELS)
			&&(adc_converters_control.enabled[channel_number]==TRUE) ){

		return_value=adc_converters_control.value[channel_number];

	}//if

	return return_value;

}//ADC_CONVERTERS_read_value



// Synth Core  13-11-2011  Tolaemon
// Function: void ADC_CONVERTERS_close()
// Function which closes ADC CONVERTERS resources
// Receives:
// Returns:
void ADC_CONVERTERS_close(){
	unsigned portCHAR uc_aux;


	if (adc_converters_control.initialized==TRUE){

		// as we are not going to use ADCs the ADC CONVERTER task is deleted
		vTaskDelete( adc_converters_control.task_handler );

		// disable the hardware of each ADC
		for (uc_aux=0;uc_aux<ADC_CONVERTERS_NUM_CHANNELS;uc_aux++){
  		    // if the ADC CONVERTER channel was enabled we disable ti
			if (adc_converters_control.enabled[uc_aux]==TRUE){
				adc_disable(adc,uc_aux);
			}//if
			adc_converters_control.value[uc_aux]=0;
			adc_converters_control.enabled[uc_aux]=FALSE;
		}//for

		// disable and reset all the vars used to control the ADC CONVERTERs
		adc_converters_control.initialized=FALSE;
		adc_converters_control.task_handler=NULL;
		adc_converters_control.sample_period=0;

	}//if

}//ADC_CONVERTERS_close
